"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor = __importStar(require("@project-serum/anchor"));
const common_1 = require("@project-serum/common");
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const spl_token_bonding_1 = require("@strata-foundation/spl-token-bonding");
const spl_utils_1 = require("@strata-foundation/spl-utils");
const bn_js_1 = __importDefault(require("bn.js"));
function mintTo(provider, mint, amount, destination) {
    return __awaiter(this, void 0, void 0, function* () {
        const ata = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mint, destination, true);
        const mintTx = new web3_js_1.Transaction();
        if (!(yield provider.connection.getAccountInfo(ata))) {
            mintTx.add(spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mint, ata, destination, provider.wallet.publicKey));
        }
        mintTx.add(spl_token_1.Token.createMintToInstruction(spl_token_1.TOKEN_PROGRAM_ID, mint, ata, provider.wallet.publicKey, [], amount));
        yield provider.send(mintTx);
    });
}
function createTestBwum(provider) {
    return __awaiter(this, void 0, void 0, function* () {
        const betaWumDest1 = new web3_js_1.PublicKey("Ge2eyjRosNwZAcQDQqJ3R4gXBy2BcpYstZYWQDUq5Rg");
        const betaWumDest2 = new web3_js_1.PublicKey("wwm872RcvN7XwNZBjXLSHfAYrFUATKgkV9v3BewHj5M");
        const splTokenMetadata = yield spl_utils_1.SplTokenMetadata.init(provider);
        const bwum = yield (0, common_1.createMint)(provider, provider.wallet.publicKey, 9);
        yield splTokenMetadata.createMetadata({
            data: new spl_utils_1.Data({
                name: 'Net bWUM Test',
                symbol: 'nbwum',
                uri: "https://wumbo-token-metadata.s3.us-east-2.amazonaws.com/bwum.json",
                creators: null,
                sellerFeeBasisPoints: 0
            }),
            mint: bwum
        });
        yield mintTo(provider, bwum, 80 * Math.pow(10, 9), betaWumDest1);
        yield mintTo(provider, bwum, 20 * Math.pow(10, 9), betaWumDest2);
        return bwum;
    });
}
function run() {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(process.env.ANCHOR_PROVIDER_URL);
        anchor.setProvider(anchor.Provider.env());
        const provider = anchor.getProvider();
        const me = provider.wallet.publicKey;
        const tokenBondingSdk = yield spl_token_bonding_1.SplTokenBonding.init(provider);
        yield tokenBondingSdk.initializeSolStorage();
        const bwum = new web3_js_1.PublicKey("HvdnoodTaRSaB7AEtm7QaDveqW9M3r4hmoNaqTggQkVp");
        console.log(`Using bwum ${bwum.toBase58()}`);
        const wSolAcct = yield spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, spl_token_1.NATIVE_MINT, me, true);
        const curve = yield tokenBondingSdk.initializeCurve({
            config: new spl_token_bonding_1.ExponentialCurveConfig({
                c: 0,
                b: 1,
                pow: 0,
                frac: 1
            })
        });
        const { tokenBonding: bonding } = yield tokenBondingSdk.createTokenBonding({
            curve,
            baseMint: spl_token_1.NATIVE_MINT,
            targetMint: bwum,
            reserveAuthority: me,
            curveAuthority: me,
            buyBaseRoyaltyPercentage: 0,
            buyTargetRoyaltyPercentage: 0,
            sellBaseRoyaltyPercentage: 0,
            sellTargetRoyaltyPercentage: 0,
            index: 1
        });
        console.log(`Created bonding ${bonding.toBase58()}`);
        const wSolBalance = (yield provider.connection.getAccountInfo(wSolAcct)).lamports;
        const bondingAcct = (yield tokenBondingSdk.getTokenBonding(bonding));
        console.log(`Transferring ${wSolBalance} lamports to the reserves`);
        const { instructions, signers, output: { destination } } = yield tokenBondingSdk.buyBondingWrappedSolInstructions({
            amount: new bn_js_1.default(wSolBalance)
        });
        instructions.unshift(spl_token_1.Token.createCloseAccountInstruction(spl_token_1.TOKEN_PROGRAM_ID, wSolAcct, me, me, []));
        instructions.push(spl_token_1.Token.createTransferInstruction(spl_token_1.TOKEN_PROGRAM_ID, destination, bondingAcct.baseStorage, me, [], wSolBalance));
        yield tokenBondingSdk.sendInstructions(instructions, signers, me);
        const reservesBalance = (yield (0, common_1.getTokenAccount)(provider, bondingAcct.baseStorage)).amount;
        console.log(`Reserves Balance: ${reservesBalance}`);
    });
}
run().catch(e => {
    console.error(e);
    process.exit(1);
});
//# sourceMappingURL=betaDist.js.map